@@ -935,50 +935,32 @@ Socket::address::address(int family, const char *a, int type, int protocol)
 	set(family, a, type, protocol);
 }
 
-Socket::address::address(const char *host, unsigned port, int family)
+Socket::address::address(const char *host, unsigned port)
 {
 	assert(host != NULL && *host != 0);
 	
-	family = setfamily(family, host);
 	list = NULL;
 #ifdef	_MSWINDOWS_
 	Socket::init();
 #endif
-	set(host, port, family);
+	set(host, port);
 }
 
-Socket::address::address(Socket &s, const char *host, const char *svc)
-{
-	assert(host != NULL && *host != 0);
-	assert(svc != NULL && *svc != 0);
-
-	list = NULL;
-
-#ifdef	_MSWINDOWS_
-	Socket::init();
-#endif
-
-	address(s.so, host, svc);
-}
-
-Socket::address::address(socket_t so, const char *host, const char *svc)
+Socket::address::address(int family, const char *host, const char *svc)
 {
 	assert(host != NULL && *host != 0);
 	assert(svc != NULL && *svc != 0);
 
 	struct addrinfo hint;
-	struct addrinfo *ah;
 
 #ifdef	_MSWINDOWS_
 	Socket::init();
 #endif
 
 	list = NULL;
-	if(so == INVALID_SOCKET)
-			return;
-
-	ah = gethint(so, &hint);
-	getaddrinfo(host, svc, ah, &list);
+	memset(&hint, 0, sizeof(hint));
+	hint.ai_family = family;
+	getaddrinfo(host, svc, &hint, &list);
 }
 
 Socket::address::address()
@@ -992,6 +974,12 @@ Socket::address::address(const address& from)
 	copy(from.list);
 }
 
+Socket::address::address(const char *host, const char *service, int type)
+{
+	list = NULL;
+	set(host, service, type);
+}
+
 Socket::address::~address()
 {
 	clear();
@@ -1084,7 +1072,7 @@ struct ::addrinfo *Socket::getaddress(const char *hp, const char *svc, int type,
 	return result;
 }	
 
-void Socket::address::set(const char *host, unsigned port, int family)
+void Socket::address::set(const char *host, unsigned port)
 {
 	assert(host != NULL && *host != 0);
 	
@@ -1225,27 +1213,23 @@ bool Socket::address::remove(struct sockaddr *addr)
 	return true;
 }
 
-unsigned Socket::address::insert(struct addrinfo *alist, int family)
+unsigned Socket::address::insert(struct addrinfo *alist)
 {
 	unsigned count = 0;
 	while(alist) {
-		if(!family || alist->ai_family == family) {
-			if(insert(alist->ai_addr))
-				++count;
-		}
+		if(insert(alist->ai_addr))
+			++count;
 		alist = alist->ai_next;
 	}
 	return count;
 }
 
-unsigned Socket::address::remove(struct addrinfo *alist, int family)
+unsigned Socket::address::remove(struct addrinfo *alist)
 {
 	unsigned count = 0;
 	while(alist) {
-		if(!family || alist->ai_family == family) {
-			if(remove(alist->ai_addr))
-				++count;
-		}
+		if(remove(alist->ai_addr))
+			++count;
 		alist = alist->ai_next;
 	}
 	return count;
@@ -1339,15 +1323,13 @@ void Socket::address::add(struct sockaddr *addr)
 	add(buffer, svc, addr->sa_family);
 }
 
-void Socket::address::set(const char *host, const char *svc, int family, int socktype)
+void Socket::address::set(const char *host, const char *svc, int socktype)
 {
-	family = setfamily(family, host);
-
 	clear();
-	add(host, svc, family, socktype);
+	add(host, svc, socktype);
 }
 
-void Socket::address::add(const char *host, const char *svc, int family, int socktype)
+void Socket::address::add(const char *host, const char *svc, int socktype)
 {
 	assert(host != NULL && *host != 0);
 	assert(svc != NULL && *svc != 0);
@@ -1396,16 +1378,22 @@ Socket::Socket(const Socket &s)
 #else
 	so = ::dup(s.so);
 #endif
+	iowait = s.iowait;
+	ioerr = 0;
 }
 
 Socket::Socket()
 {
 	so = INVALID_SOCKET;
+	iowait = Timer::inf;
+	ioerr = 0;
 }
 
 Socket::Socket(socket_t s)
 {
 	so = s;
+	iowait = Timer::inf;
+	ioerr = 0;
 }
 
 Socket::Socket(struct addrinfo *addr)
@@ -1425,14 +1413,18 @@ Socket::Socket(struct addrinfo *addr)
 		addr = addr->ai_next;
 	}
 	so = INVALID_SOCKET;
+	iowait = Timer::inf;
+	ioerr = 0;
 }
 
 Socket::Socket(int family, int type, int protocol)
 {
 	so = create(family, type, protocol);
+	iowait = Timer::inf;
+	ioerr = 0;
 }
 
-Socket::Socket(const char *iface, const char *port, int family, int type, int protocol, int backlog)
+Socket::Socket(const char *iface, const char *port, int family, int type, int protocol)
 {
 	assert(iface != NULL && *iface != 0);
 	assert(port != NULL && *port != 0);
@@ -1441,7 +1433,9 @@ Socket::Socket(const char *iface, const char *port, int family, int type, int pr
 	init();
 #endif
 	family = setfamily(family, iface);
-	so = create(iface, port, family, type, protocol, 0);
+	so = create(iface, port, family, type, protocol);
+	iowait = Timer::inf;
+	ioerr = 0;
 }
 
 socket_t Socket::create(Socket::address &address)
@@ -1462,7 +1456,7 @@ socket_t Socket::create(Socket::address &address)
 	return so;
 }
 
-socket_t Socket::create(const char *iface, const char *port, int family, int type, int protocol, int backlog)
+socket_t Socket::create(const char *iface, const char *port, int family, int type, int protocol)
 {
 	assert(iface != NULL && *iface != 0);
 	assert(port != NULL && *port != 0);
@@ -1499,10 +1493,6 @@ socket_t Socket::create(const char *iface, const char *port, int family, int typ
 			release(so);
 			return INVALID_SOCKET;
 		}
-		if(type == SOCK_STREAM && _listen_(so, backlog)) {
-			release(so);
-			return INVALID_SOCKET;
-		}
 		return so;	
 	};
 #endif
@@ -1513,6 +1503,7 @@ socket_t Socket::create(const char *iface, const char *port, int family, int typ
 	getaddrinfo(iface, port, &hint, &res);
 	if(res == NULL)
 		return INVALID_SOCKET;
+
 	so = create(res->ai_family, res->ai_socktype, res->ai_protocol);
 	if(so == INVALID_SOCKET) {
 		freeaddrinfo(res);
@@ -1524,12 +1515,6 @@ socket_t Socket::create(const char *iface, const char *port, int family, int typ
 			release(so);
 			so = INVALID_SOCKET;
 		}
-		else if(res->ai_socktype == SOCK_STREAM) {
-			if(_listen_(so, backlog)) {
-				release(so);
-				so = INVALID_SOCKET;
-			}
-		}
 	}
 	freeaddrinfo(res);
 	return so;
@@ -1584,6 +1569,8 @@ void Socket::release(void)
 #endif
 		so = INVALID_SOCKET;
 	}
+	iowait = Timer::inf;
+	ioerr = 0;
 }
 
 #ifdef	_MSWINDOWS_
@@ -1688,6 +1675,9 @@ size_t Socket::peek(void *data, size_t len) const
 	assert(data != NULL);
 	assert(len > 0);
 
+	if(iowait && iowait != Timer::inf && !Socket::wait(so, iowait))
+		return 0;
+
 	ssize_t rtn = _recv_(so, (caddr_t)data, 1, MSG_DONTWAIT | MSG_PEEK);
 	if(rtn < 1)
 		return 0;
@@ -1712,16 +1702,26 @@ ssize_t Socket::recvfrom(socket_t so, void *data, size_t len, int flags, struct
 	return _recvfrom_(so, (caddr_t)data, len, flags, (struct sockaddr *)addr, &slen);
 }
 
-ssize_t Socket::get(void *data, size_t len, struct sockaddr_storage *from)
+size_t Socket::readfrom(void *data, size_t len, struct sockaddr_storage *from)
 {
 	assert(data != NULL);
 	assert(len > 0);
 
+	// wait for input by timer if possible...
+	if(iowait && iowait != Timer::inf && !Socket::wait(so, iowait))
+		return 0;
+
 	socklen_t slen = sizeof(struct sockaddr_storage);
-	return _recvfrom_(so, (caddr_t)data, len, 0, (struct sockaddr *)from, &slen);
+	ssize_t result = _recvfrom_(so, (caddr_t)data, len, 0, (struct sockaddr *)from, &slen);
+
+	if(result < 0) {
+		ioerr = Socket::error();
+		return 0;
+	}
+	return (size_t)result;		
 }
 
-ssize_t Socket::put(const void *data, size_t len, struct sockaddr *dest)
+size_t Socket::writeto(const void *data, size_t len, struct sockaddr *dest)
 {
 	assert(data != NULL);
 	assert(len > 0);
@@ -1730,7 +1730,12 @@ ssize_t Socket::put(const void *data, size_t len, struct sockaddr *dest)
 	if(dest)
 		slen = getlen(dest);
 	
-	return _sendto_(so, (caddr_t)data, len, MSG_NOSIGNAL, dest, slen);
+	ssize_t result = _sendto_(so, (caddr_t)data, len, MSG_NOSIGNAL, dest, slen);
+	if(result < 0) {
+		ioerr = Socket::error();
+		return 0;
+	}
+	return (size_t)result;
 }
 
 ssize_t Socket::sendto(socket_t so, const void *data, size_t len, int flags, struct sockaddr *dest)
@@ -1745,7 +1750,7 @@ ssize_t Socket::sendto(socket_t so, const void *data, size_t len, int flags, str
 	return _sendto_(so, (caddr_t)data, len, MSG_NOSIGNAL | flags, dest, slen);
 }
 
-ssize_t Socket::puts(const char *str, struct sockaddr *address)
+size_t Socket::writes(const char *str)
 {
 	if(!str)
 		return 0;
@@ -1753,23 +1758,37 @@ ssize_t Socket::puts(const char *str, struct sockaddr *address)
 	if(!*str)
 		return 0;
 
-	return put(str, strlen(str), address);
+	return writeto(str, strlen(str), NULL);
 }
 
-ssize_t Socket::puts(const char *str)
+size_t Socket::readline(char *data, size_t max)
 {
-	if(!str)
-		return 0;
+	assert(data != NULL);
+	assert(max > 0);
 
-	if(!*str)
-		return 0;
+	*data = 0;
 
-	return put(str, strlen(str));
+	ssize_t result = Socket::readline(so, data, max, iowait);
+	if(result < 0) {
+		ioerr = Socket::error();
+		return 0;
+	}
+	return (size_t)result;
 }
 
-ssize_t Socket::gets(char *data, size_t max, timeout_t timeout)
+size_t Socket::readline(string& s)
 {
-	return Socket::readline(so, data, max, timeout);
+	if(!s.c_mem())
+		return 0;
+
+	ssize_t result = Socket::readline(so, s.c_mem(), s.size() + 1, iowait);
+	if(result < 0) {
+		ioerr = Socket::error();
+		s.clear();
+		return 0;
+	}
+	String::fix(s);
+	return (size_t)result;
 }
 
 ssize_t Socket::readline(socket_t so, char *data, size_t max, timeout_t timeout)
@@ -1789,11 +1808,14 @@ ssize_t Socket::readline(socket_t so, char *data, size_t max, timeout_t timeout)
 	while(nleft && !nl) {
 		if(timeout) {
 			if(!wait(so, timeout))
-				return -1;
+				return 0;
 		}
 		nstat = _recv_(so, data, nleft, MSG_PEEK);
-		if(nstat <= 0)
+		if(nstat < 0)
 			return -1;
+
+		if(nstat == 0)
+			return max - nleft - 1;
 		
 		for(c = 0; c < nstat; ++c) {
 			if(data[c] == '\n') {
@@ -1804,6 +1826,7 @@ ssize_t Socket::readline(socket_t so, char *data, size_t max, timeout_t timeout)
 				break;
 			}
 		}
+
 		nstat = _recv_(so, (caddr_t)data, c, 0);
 		if(nstat < 0)
 			break;
@@ -1816,6 +1839,10 @@ ssize_t Socket::readline(socket_t so, char *data, size_t max, timeout_t timeout)
 		data += nstat;
 		nleft -= nstat;
 	}
+
+	if(nl)
+		--data;
+
 	*data = 0;
 	return ssize_t(max - nleft - 1);
 }
@@ -1836,19 +1863,25 @@ int Socket::loopback(socket_t so, bool enable)
 		opt = 1;
 
 	if(so == INVALID_SOCKET)
-		return -1;
+		return EBADF;
 
 	_getsockname_(so, addr, &len);
 	family = us.inaddr.sin_family;
 	switch(family) {
 	case AF_INET:
-		return setsockopt(so, IPPROTO_IP, IP_MULTICAST_LOOP, (char *)&opt, sizeof(opt));
+		if(!setsockopt(so, IPPROTO_IP, IP_MULTICAST_LOOP, (char *)&opt, sizeof(opt)))
+			return 0;
+		break;
 #if defined(AF_INET6) && defined(IPROTO_IPV6)
 	case AF_INET6:
-		return setsockopt(so, IPPROTO_IPV6, IPV6_MULTICAST_LOOP, (char *)&opt, sizeof(opt));
+		if(!setsockopt(so, IPPROTO_IPV6, IPV6_MULTICAST_LOOP, (char *)&opt, sizeof(opt)))
+			return 0;
 #endif
 	}
-	return -1;
+	int err = Socket::error();
+	if(!err)
+		err = EIO;
+	return err;
 }
 
 int Socket::ttl(socket_t so, unsigned char t)
@@ -1863,74 +1896,108 @@ int Socket::ttl(socket_t so, unsigned char t)
 	socklen_t len = sizeof(us.saddr);
 
 	if(so == INVALID_SOCKET)
-		return -1;
+		return EBADF;
 
 	_getsockname_(so, addr, &len);
 	family = us.inaddr.sin_family;
 	switch(family) {
 	case AF_INET:
-		return setsockopt(so, IPPROTO_IP, IP_TTL, (char *)&t, sizeof(t));
+		if(!setsockopt(so, IPPROTO_IP, IP_TTL, (char *)&t, sizeof(t)))
+			return 0;
+		break;
 #if defined(AF_INET6) && defined(IPPROTO_IPV6)
 	case AF_INET6:
-		return setsockopt(so, IPPROTO_IPV6, IPV6_UNICAST_HOPS, (char *)&t, sizeof(t));
+		if(!setsockopt(so, IPPROTO_IPV6, IPV6_UNICAST_HOPS, (char *)&t, sizeof(t)))
+			return 0;
 #endif
 	}
-	return -1;
+	int err = Socket::error();
+	if(!err)
+		err = EIO;
+	return err;
 }
 
 int Socket::priority(socket_t so, int pri)
 {
+	if(so == INVALID_SOCKET)
+		return EBADF;
 #ifdef	SO_PRIORITY
-	return setsockopt(so, SOL_SOCKET, SO_PRIORITY, (char *)&pri, (socklen_t)sizeof(pri));
+	if(!setsockopt(so, SOL_SOCKET, SO_PRIORITY, (char *)&pri, (socklen_t)sizeof(pri)))
+		return 0;
+	int err = Socket::error();
+	if(!err)
+		err = EIO;
+	return err;
 #else
-	return -1;
+	return ENOSYS;
 #endif
 }
 
 int Socket::tos(socket_t so, int ts)
 {
 	if(so == INVALID_SOCKET)
-		return -1;
+		return EBADF;
 
 #ifdef	SOL_IP
-	return setsockopt(so, SOL_IP, IP_TOS,(char *)&ts, (socklen_t)sizeof(ts));
+	if(!setsockopt(so, SOL_IP, IP_TOS,(char *)&ts, (socklen_t)sizeof(ts)))
+		return 0;
+
+	int err = Socket::error();
+	if(!err)
+		err = EIO;
+	return err;
 #else
-	return -1;
+	return ENOSYS;
 #endif
 }
 
 int Socket::broadcast(socket_t so, bool enable)
 {
 	if(so == INVALID_SOCKET)
-		return -1;
+		return EBADF;
     int opt = (enable ? 1 : 0);
-    return ::setsockopt(so, SOL_SOCKET, SO_BROADCAST,
-              (char *)&opt, (socklen_t)sizeof(opt));
+    if(!::setsockopt(so, SOL_SOCKET, SO_BROADCAST,
+              (char *)&opt, (socklen_t)sizeof(opt)))
+		return 0;
+
+	int err = Socket::error();
+	if(!err)
+		err = EIO;
+	return err;
 }
 
 int Socket::nodelay(socket_t so)
 {
 	if(so == INVALID_SOCKET)
-		return -1;
+		return EBADF;
 #if defined(TCP_NODELAY)
 	int opt = 1;
-	return ::setsockopt(so, IPPROTO_TCP, TCP_NODELAY,
-		(char *)&opt, (socklen_t)sizeof(opt));
+	if(!::setsockopt(so, IPPROTO_TCP, TCP_NODELAY,
+			(char *)&opt, (socklen_t)sizeof(opt)))
+		return 0;
 #else
-	return -1;
+	return ENOSYS;
 #endif
+	int err = Socket::error();
+	if(!err)
+		err = EIO;
+	return err;
 }
 
 int Socket::keepalive(socket_t so, bool enable)
 {
 	if(so == INVALID_SOCKET)
-		return -1;
+		return EBADF;
 #if defined(SO_KEEPALIVE) || defined(_MSWINDOWS_)
 	int opt = (enable ? ~0 : 0);
-	return ::setsockopt(so, SOL_SOCKET, SO_KEEPALIVE,
-		(char *)&opt, (socklen_t)sizeof(opt));
+	if(!::setsockopt(so, SOL_SOCKET, SO_KEEPALIVE, (char *)&opt, (socklen_t)sizeof(opt)))
+		return 0;
+	int err = Socket::error();
+	if(!err)
+		err = EIO;
+	return err;
 #else
-	return -1;
+	return ENOSYS;
 #endif
 }				
 
@@ -1942,7 +2009,7 @@ int Socket::multicast(socket_t so, unsigned ttl)
 	int rtn;
 
 	if(so == INVALID_SOCKET)
-		return -1;
+		return EBADF;
 
 	if(ttl)
 		enable = true;
@@ -1968,40 +2035,89 @@ int Socket::multicast(socket_t so, unsigned ttl)
 #if defined(AF_INET6) && defined(IPPROTO_IPV6)
 	case AF_INET6:
 		rtn = ::setsockopt(so, IPPROTO_IPV6, IPV6_MULTICAST_IF, (char *)&addr.ipv6.sin6_addr, sizeof(addr.ipv6.sin6_addr));
-		if(rtn)
-			return rtn;
-		return ::setsockopt(so, IPPROTO_IPV6, IPV6_MULTICAST_HOPS, (char *)&ttl, sizeof(ttl));
+		if(!rtn)
+			rtn = ::setsockopt(so, IPPROTO_IPV6, IPV6_MULTICAST_HOPS, (char *)&ttl, sizeof(ttl));
+		if(rtn) {
+			rtn = Socket::error();
+			if(!rtn)
+				rtn = EIO;
+		}
+		return rtn;
 #endif
 	case AF_INET:
 #ifdef	IP_MULTICAST_IF
 		rtn = ::setsockopt(so, IPPROTO_IP, IP_MULTICAST_IF, (char *)&addr.ipv4.sin_addr, sizeof(addr.ipv4.sin_addr));
-		if(rtn)
-			return rtn;
-		return setsockopt(so, IPPROTO_IP, IP_MULTICAST_TTL, (char *)&ttl, sizeof(ttl));
+		if(!rtn)
+			rtn = ::setsockopt(so, IPPROTO_IP, IP_MULTICAST_TTL, (char *)&ttl, sizeof(ttl));
+		if(rtn) {
+			rtn = Socket::error();
+			if(!rtn)
+				rtn = EIO;
+		}
+		return rtn;
 #else
-		return -1;
+		return ENOSYS;
 #endif
 	default:
-		return -1;
+		return ENOSYS;
 	}	
 }
 
+int Socket::join(struct addrinfo *addr)
+{
+	int rtn = Socket::join(so, addr);
+	if(rtn)
+		ioerr = rtn;
+	return rtn;
+}
+
+int Socket::drop(struct addrinfo *addr)
+{
+	int rtn = Socket::drop(so, addr);
+	if(rtn)
+		ioerr = rtn;
+	return rtn;
+}
+
+int Socket::wait(timeout_t timeout)
+{
+	bool mode = true;
+
+	if(timeout < Timer::inf)
+		mode = false;
+
+	int rtn = Socket::blocking(so, mode);
+	if(!rtn)
+		iowait = timeout;
+	else
+		ioerr = rtn;
+
+	return rtn;
+}
+
 int Socket::blocking(socket_t so, bool enable)
 {
 	if(so == INVALID_SOCKET)
-		return -1;
+		return EBADF;
 
 #if defined(_MSWINDOWS_)
 	unsigned long flag = (enable ? 0 : 1);
-	return ioctlsocket(so, FIONBIO, &flag);
+	if(!ioctlsocket(so, FIONBIO, &flag))
+		return 0;
+	
 #else
 	long flags = fcntl(so, F_GETFL);
 	if(enable)
 		flags &=~ O_NONBLOCK;
 	else
 		flags |= O_NONBLOCK;
-	return fcntl(so, F_SETFL, flags);
+	if(!fcntl(so, F_SETFL, flags))
+		return 0;
 #endif
+	int err = Socket::error();
+	if(!err)
+		err = EIO;
+	return err;
 }
 
 int Socket::disconnect(socket_t so)
@@ -2025,7 +2141,14 @@ int Socket::disconnect(socket_t so)
 #endif
 	if(len > sizeof(us.saddr))
 		len = sizeof(us.saddr);
-	return _connect_(so, addr, len);
+	if(so == INVALID_SOCKET)
+		return EBADF;
+	if(!_connect_(so, addr, len))
+		return 0;
+	int err = Socket::error();
+	if(!err)
+		err = EIO;
+	return err;
 }
 
 int Socket::join(socket_t so, struct addrinfo *node)
@@ -2040,7 +2163,7 @@ int Socket::join(socket_t so, struct addrinfo *node)
 	int rtn = 0;
 
 	if(so == INVALID_SOCKET)
-		return -1;
+		return EBADF;
 	
 	_getsockname_(so, (struct sockaddr *)&addr, &len);
 	while(!rtn && node && node->ai_addr) {
@@ -2065,9 +2188,14 @@ int Socket::join(socket_t so, struct addrinfo *node)
 			break;
 #endif
 		default:
-			rtn = -1;
+			return ENOSYS;
 		}
 	}
+	if(rtn) {
+		rtn = Socket::error();
+		if(!rtn)
+			rtn = EIO;
+	}
 	return rtn;
 }
 
@@ -2083,7 +2211,7 @@ int Socket::drop(socket_t so, struct addrinfo *node)
 	int rtn = 0;
 
 	if(so == INVALID_SOCKET)
-		return -1;
+		return EBADF;
 	
 	_getsockname_(so, (struct sockaddr *)&addr, &len);
 	while(!rtn && node && node->ai_addr) {
@@ -2110,9 +2238,14 @@ int Socket::drop(socket_t so, struct addrinfo *node)
 			break;
 #endif
 		default:
-			rtn = -1;
+			return ENOSYS;
 		}
 	}
+	if(rtn) {
+		rtn = Socket::error();
+		if(!rtn)
+			rtn = EIO;
+	}
 	return rtn;
 }
 
@@ -2156,7 +2289,17 @@ next:
 		Socket::release(so);
 	return INVALID_SOCKET;
 }
-	
+
+int Socket::connectto(struct addrinfo *node)
+{
+	return (ioerr = connectto(so, node));
+}
+
+int Socket::disconnect(void)
+{
+	return (ioerr = disconnect(so));
+}
+
 int Socket::connectto(socket_t so, struct addrinfo *node)
 {
 	assert(node != NULL);
@@ -2165,7 +2308,7 @@ int Socket::connectto(socket_t so, struct addrinfo *node)
 	int socket_family;
 	
 	if(so == INVALID_SOCKET)
-		return -1;
+		return EBADF;
 
 	socket_family = getfamily(so);
 
@@ -2184,6 +2327,11 @@ exit:
 	if(!rtn || errno == EINPROGRESS)
 		return 0;
 #endif
+	if(rtn) {
+		rtn = Socket::error();
+		if(!rtn)
+			rtn = EIO;
+	}
 	return rtn;
 }
 
@@ -2195,7 +2343,7 @@ int Socket::error(socket_t so)
 	socklen_t slen = sizeof(opt);
 
 	if(getsockopt(so, SOL_SOCKET, SO_ERROR, (caddr_t)&opt, &slen))
-		return -1;
+		return ENOSYS;
 	
 	return opt;
 }
@@ -2205,9 +2353,14 @@ int Socket::sendwait(socket_t so, unsigned size)
 	assert(so != INVALID_SOCKET);
 
 #ifdef	SO_SNDLOWAT
-	return setsockopt(so, SOL_SOCKET, SO_SNDLOWAT, (caddr_t)&size, sizeof(size));
+	if(!setsockopt(so, SOL_SOCKET, SO_SNDLOWAT, (caddr_t)&size, sizeof(size)))
+		return 0;
+	int err = Socket::error();
+	if(!err)
+		err = EIO;
+	return err;
 #else
-	return -1;
+	return ENOSYS;
 #endif
 }
 
@@ -2216,18 +2369,28 @@ int Socket::sendsize(socket_t so, unsigned size)
 	assert(so != INVALID_SOCKET);
 
 #ifdef	SO_SNDBUF
-	return setsockopt(so, SOL_SOCKET, SO_SNDBUF, (caddr_t)&size, sizeof(size));
+	if(!setsockopt(so, SOL_SOCKET, SO_SNDBUF, (caddr_t)&size, sizeof(size)))
+		return 0;
+	int err = Socket::error();
+	if(!err)
+		err = EIO;
+	return err;
 #else
-	return -1;
+	return ENOSYS;
 #endif
 }
 
 int Socket::recvsize(socket_t so, unsigned size)
 {
 #ifdef	SO_RCVBUF
-	return setsockopt(so, SOL_SOCKET, SO_RCVBUF, (caddr_t)&size, sizeof(size));
+	if(!setsockopt(so, SOL_SOCKET, SO_RCVBUF, (caddr_t)&size, sizeof(size)))
+		return 0;
+	int err = Socket::error();
+	if(!err)
+		err = EIO;
+	return err;
 #else
-	return -1;
+	return ENOSYS;
 #endif
 }
 
@@ -2401,47 +2564,37 @@ bool Socket::waitSending(timeout_t timeout) const
 #endif
 }
 
-ListenSocket::ListenSocket(const char *iface, const char *svc, unsigned backlog, int protocol) :
+ListenSocket::ListenSocket(const char *iface, const char *svc, unsigned backlog, int family, int type, int protocol) :
 Socket()
 {
+	if(!iface)
+		iface = "*";
+
 	assert(iface != NULL && *iface != 0);
 	assert(svc != NULL && *svc != 0);
 	assert(backlog > 0);
-
-	int family = AF_INET;
-
-	if(strchr(iface, '/'))
-		family = AF_UNIX;
-#ifdef	AF_INET6
-	else if(strchr(iface, ':'))
-		family = AF_INET6;
-#endif
-
-retry:
-	if(protocol == IPPROTO_DCCP)
-		so = ::socket(family, SOCK_DCCP, protocol);
-	else
-		so = ::socket(family, SOCK_STREAM, protocol);
-
-	if(so == INVALID_SOCKET)
-		return;
-		
-	socket_mapping(family, so);
-	if(bindto(so, iface, svc, protocol)) {
-		release();
-#ifdef	AF_INET6
-		if(family == AF_INET && !strchr(iface, '.')) {
-			family = AF_INET6;
-			goto retry;
-		}
-#endif
-		return;
-	}
-	if(_listen_(so, backlog))
-		release();
+	
+	so = create(iface, svc, backlog, family, type, protocol);
 }
 
-socket_t ListenSocket::accept(struct sockaddr_storage *addr)
+socket_t ListenSocket::create(const char *iface, const char *svc, unsigned backlog, int family, int type, int protocol)
+{
+	if(!type)
+		type = SOCK_STREAM;
+
+	socket_t so = Socket::create(iface, svc, family, type, protocol);
+
+	if(so == INVALID_SOCKET)
+		return so;
+
+	if(_listen_(so, backlog)) {
+		release(so);
+		return INVALID_SOCKET;
+	}
+	return so;
+}
+
+socket_t ListenSocket::accept(struct sockaddr_storage *addr) const
 {
 	socklen_t len = sizeof(struct sockaddr_storage);
 	if(addr)
@@ -2450,6 +2603,11 @@ socket_t ListenSocket::accept(struct sockaddr_storage *addr)
 		return _accept_(so, NULL, NULL);
 }
 
+TCPServer::TCPServer(const char *address, const char *service, unsigned backlog) :
+ListenSocket(address, service, backlog)
+{
+}
+
 #ifdef	_MSWINDOWS_
 #undef	AF_UNIX
 #endif
@@ -2610,14 +2768,18 @@ exit:
 
 int Socket::bindto(socket_t so, struct sockaddr *iface)
 {
-	return _bind_(so, iface, getlen(iface));
+	if(!_bind_(so, iface, getlen(iface)))
+		return 0;
+	return Socket::error();
 }
 
 int Socket::listento(socket_t so, struct sockaddr *iface, int backlog)
 {
 	if(_bind_(so, iface, getlen(iface)))
-		return -1;
-	return _listen_(so, backlog);
+		return Socket::error();
+	if(_listen_(so, backlog))
+		return Socket::error();
+	return 0;
 }
 
 int Socket::bindto(socket_t so, const char *host, const char *svc, int protocol)
@@ -2643,7 +2805,7 @@ int Socket::bindto(socket_t so, const char *host, const char *svc, int protocol)
 #endif
 
     if(!gethint(so, &hint) || !svc)
-        return -1;
+        return ENOSYS;
 
 	hint.ai_protocol = protocol;
 	if(host && !strcmp(host, "*"))
@@ -2674,6 +2836,8 @@ int Socket::bindto(socket_t so, const char *host, const char *svc, int protocol)
 exit:
 	if(res)
 		freeaddrinfo(res);
+	if(rtn)
+		rtn = Socket::error();
 	return rtn;
 }
 
@@ -2834,7 +2998,7 @@ int Socket::getinterface(struct sockaddr *iface, struct sockaddr *dest)
 			rtn = _getsockname_(so, iface, &len);
 		break;
 	default:
-		return -1;
+		return ENOSYS;
 	}
 	switch(iface->sa_family) {
 	case AF_INET:
@@ -2856,6 +3020,8 @@ int Socket::getinterface(struct sockaddr *iface, struct sockaddr *dest)
 #endif
 		so = INVALID_SOCKET;
 	}
+	if(rtn)
+		rtn = Socket::error();
 	return rtn;
 }
 
@@ -2997,6 +3163,20 @@ bool Socket::equal(struct sockaddr *s1, struct sockaddr *s2)
 	return false;
 }
 
+size_t Socket::printf(const char *format, ...)
+{
+    assert(format != NULL);
+
+    char buf[1024];
+    va_list args;
+
+    va_start(args, format);
+    vsnprintf(buf, sizeof(buf), format, args);
+    va_end(args);
+
+    return writes(buf);
+}
+
 ssize_t Socket::printf(socket_t so, const char *format, ...)
 {
 	assert(format != NULL);
@@ -3047,74 +3227,6 @@ int Socket::getfamily(socket_t so)
 	return us.inaddr.sin_family;
 }
 
-#ifdef	_MSWINDOWS_
-
-FILE *Socket::open(socket_t so)
-{
-	FILE *fp = (FILE *)malloc(sizeof(FILE));
-
-	if(!fp)
-		return NULL;
-
-	memset(fp, 0, sizeof(FILE));
-	fp->_file = so;
-	fp->_flag = _IOWRT | _IOREAD;
-	return fp;
-}
-
-FILE *Socket::open(socket_t so, bool mode)
-{
-	FILE *fp = (FILE *)malloc(sizeof(FILE));
-
-	if(!fp)
-		return NULL;
-
-	memset(fp, 0, sizeof(FILE));
-
-	if(mode) {
-		fp->_file = so;
-		fp->_flag = _IOWRT;
-	}
-	else {
-		fp->_file = so;
-		fp->_flag = _IOREAD;
-	}
-	return fp;
-}
-
-void Socket::close(FILE *fp)
-{
-	assert(fp != NULL);
-
-	if((fp->_flag & (_IOREAD | _IOWRT))== _IOREAD) {
-		::shutdown(fp->_file, SHUT_RDWR);
-		closesocket(fp->_file);
-	}
-	free(fp);
-}	
-
-#else
-
-void Socket::close(FILE *fp)
-{
-	fclose(fp);
-}
-
-FILE *Socket::open(socket_t so)
-{
-	return fdopen(so, "rw");
-}
-
-FILE *Socket::open(socket_t so, bool mode)
-{
-	if(mode)
-		return fdopen(so, "w");
-
-	return fdopen(dup(so), "r");
-}
-
-#endif
-
 bool Socket::isNull(const char *str)
 {
 	assert(str != NULL);
@@ -3217,3 +3329,10 @@ void Socket::release(set_t mask)
 	free(mask);
 }
 
+String str(Socket& so, strsize_t size)
+{
+	String s(size);
+	so.readline(s.c_mem(), s.size());
+	String::fix(s);
+	return s;
+}
