@@ -17,8 +17,6 @@
 
 #include <config.h>
 #include <ucommon/string.h>
-#include <ucommon/thread.h>
-#include <ucommon/socket.h>
 #include <stdarg.h>
 #include <ctype.h>
 #include <stdio.h>
@@ -30,10 +28,6 @@ using namespace UCOMMON_NAMESPACE;
 const strsize_t string::npos = (strsize_t)(-1);
 const size_t memstring::header = sizeof(cstring);
 
-StringFormat::~StringFormat()
-{
-}
-
 string::cstring::cstring(strsize_t size) :
 CountedObject()
 {
@@ -211,24 +205,6 @@ strsize_t string::len(void)
 	return str->len;
 }
 
-void string::cstring::add(const StringFormat& format)
-{
-	unfix();
-	if(len < max - 1)
-		format.put(text + len, max - len);
-	text[max - 1] = 0;
-	len = strlen(text);
-	fix();
-}
-
-void string::cstring::set(const StringFormat& format)
-{
-	format.put(text, max);
-	text[max - 1] = 0;
-	len = strlen(text);
-	fix();
-}
-
 void string::cstring::set(const char *str)
 {
 	assert(str != NULL);
@@ -275,13 +251,6 @@ string::string(const char *s)
 	str->set(s);
 }
 
-string::string(const StringFormat& f)
-{
-	str = create(f.getStringSize());
-	str->retain();
-	str->set(f);
-}
-
 string::string(const char *s, strsize_t size)
 {
 	if(!s)
@@ -821,13 +790,6 @@ void string::rset(const char *s, char overflow, strsize_t offset, strsize_t size
 		str->text[offset] = overflow;
 }
 
-void string::set(const StringFormat& f)
-{
-	resize(f.getStringSize());
-	assert(str);
-	str->set(f);
-}
-
 void string::set(const char *s)
 {
 	strsize_t len;
@@ -925,14 +887,6 @@ void string::cow(strsize_t size)
 	}
 }
 
-void string::add(const StringFormat& f)
-{
-	if(!str)
-		set(f);
-	else
-		str->add(f);
-}
-
 void string::add(const char *s)
 {
 	if(!s || !*s)
@@ -1044,19 +998,6 @@ string &string::operator^=(const char *s)
 	return *this;
 }
 
-string &string::operator^=(const StringFormat& f)
-{
-	release();
-	set(f);
-	return *this;
-}
-
-string &string::operator=(const StringFormat& f)
-{
-	set(f);
-	return *this;
-}
-
 string &string::operator=(const char *s)
 {
 	set(s);
@@ -1155,25 +1096,12 @@ bool string::operator>=(const char *s) const
     return (compare(s) >= 0);
 }
 
-string &string::operator&(const StringFormat& f)
-{
-	add(f);
-	return *this;
-}
-
 string &string::operator&(const char *s)
 {
 	add(s);
 	return *this;
 }
 
-string &string::operator+(const StringFormat& f)
-{
-	cow(f.getStringSize());
-	add(f);
-	return *this;
-}
-
 string &string::operator+(const char *s)
 {
 	if(!s || !*s)
@@ -1206,7 +1134,7 @@ memstring *memstring::create(strsize_t size, char fill)
 	return new(mem) memstring(mem + sizeof(memstring), size, fill);
 }
 
-memstring *memstring::create(mempager *mpager, strsize_t size, char fill)
+memstring *memstring::create(MemoryProtocol *mpager, strsize_t size, char fill)
 {
 	assert(size > 0);
 
@@ -1247,7 +1175,7 @@ char *string::token(char *text, char **token, const char *clist, const char *quo
 	}
 
 	while(**token && strchr(clist, **token))
-		++token;
+		++*token;
 
 	result = *token;
 
@@ -1336,122 +1264,6 @@ void string::swap(string &s1, string &s2)
 	s1.str = s2.str;
 	s2.str = s;
 }
-
-int string::read(Socket &so, string &s)
-{
-	int rtn;
-
-	if(!mem(s))
-		return 0;
-
-	clear(s);
-
-	rtn = so.get(mem(s), size(s));
-	if(rtn > -1) {
-		s.str->len = rtn;
-		s.str->text[rtn] = 0;
-	}
-	return rtn;
-}
-
-int string::write(Socket &so, string &s)
-{
-	if(!mem(s))
-		return 0;
-
-	return so.put(mem(s), len(s));
-}
-
-int string::read(FILE *fp, string &s)
-{
-	int rtn;
-
-	if(!mem(s))
-		return 0;
-
-	clear(s);
-
-	rtn = fread(mem(s), 1, size(s), fp);
-	if(rtn > -1) {
-		s.str->len = rtn;
-		s.str->text[rtn] = 0;
-	}
-	return rtn;
-}
-
-int string::write(FILE *fp, string &s)
-{
-	assert(fp != NULL);
-
-	if(!mem(s))
-		return 0;
-
-	return fwrite(mem(s), 1, len(s), fp);
-}
-
-bool string::putline(Socket &so, string &s)
-{
-	if(s[-1] != '\n')
-		s.add("\r\n");
-		
-	if(so.puts(mem(s)) >= 0)
-		return true;
-
-	return false;
-}
-
-bool string::putline(FILE *fp, string &s)
-{
-	assert(fp != NULL);
-
-	if(s[-1] != '\n')
-		s.add("\n");
-		
-	if(fputs(mem(s), fp) >= 0)
-		return true;
-
-	return false;
-}
-
-bool string::getline(Socket &so, string &s)
-{
-	if(!mem(s))
-		return true;
-
-	if(so.gets(mem(s), size(s)) < 0) 
-		return false;
-
-	fix(s);
-
-	if(s[-1] == '\n')
-		--s;
-
-	if(s[-1] == '\r')
-		--s;
-
-	return true;
-}
-
-bool string::getline(FILE *fp, string &s)
-{
-	assert(fp != NULL);
-
-	if(!mem(s))
-		return true;
-
-	if(!::fgets(mem(s), size(s), fp) || feof(fp)) {
-		clear(s);
-		return false;
-	}
-	fix(s);
-	if(s[-1] == '\n')
-		--s;
-
-	if(s[-1] == '\r')
-		--s;
-
-	return true;
-}
 	
 char *string::dup(const char *cp)
 {
@@ -2085,5 +1897,32 @@ string &string::operator%(const char *get)
 	return *this;
 }
 
+String str(CharacterProtocol *p, strsize_t size)
+{
+	String temp(size);
+	char *cp = temp.c_mem();
+	int ch;
+	bool cr = false;
+
+	while(--size) {
+		ch = p->get();
+		if(ch == 0 || ch == EOF || ch == '\n')
+			break;
+
+		if(cr) {
+			cr = false;
+			*(cp++) = '\r';
+		}
+
+		if(ch == '\r')
+			cr = true;
+		else
+			*(cp++) = ch;
+	}
+
+	*cp = 0;
+
+	String::fix(temp);
+	return temp;
+}
 
-	
