@@ -85,7 +85,9 @@ public:
 	/**
 	 * Convenience type to represent a security context.
 	 */
-	typedef	secure *context_t;
+	typedef	secure *client_t;
+
+	typedef secure *server_t;
 
 	/**
 	 * Convenience type to represent a secure socket session.
@@ -126,7 +128,7 @@ public:
 	 * @param authority path to use or NULL if none.
 	 * @return a security context that is cast from derived library.
 	 */
-	static context_t server(const char *authority = NULL);
+	static server_t server(const char *authority = NULL);
 
 	/**
 	 * Create an anonymous client context with an optional authority to 
@@ -134,7 +136,7 @@ public:
 	 * @param authority path to use or NULL if none.
 	 * @return a basic client security context.
 	 */
-	static context_t client(const char *authority = NULL);
+	static client_t client(const char *authority = NULL);
 
 	/**
 	 * Create a peer user client context.  This assumes a user certificate
@@ -142,14 +144,14 @@ public:
 	 * path to an authority is also sent.
 	 * @param authority path to use.
 	 */
-	static context_t user(const char *authority);
+	static client_t user(const char *authority);
 
 	/**
 	 * Assign a non-default cipher to the context.
 	 * @param context to set cipher for.
 	 * @param ciphers to set.
 	 */
-	static void cipher(context_t context, const char *ciphers);
+	static void cipher(secure *context, const char *ciphers);
 
 	/**
 	 * Determine if the current security context is valid.
@@ -175,47 +177,47 @@ public:
 };
 
 /**
- * Secure socket class.  This is used to create ssl socket connections
+ * Secure socket buffer.  This is used to create ssl socket connections
  * for both clients and servers.  The use depends in part on the type of
  * context created and passed at construction time.  If no context is
- * passed (NULL), then this reverts to TCPSocket behavior.
+ * passed (NULL), then this reverts to TCPBuffer behavior.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-class __EXPORT SSocket : public TCPSocket
+class __EXPORT SSLBuffer : public TCPBuffer
 {
 protected:
 	secure::session_t ssl;
 	secure::bufio_t bio;
+	bool server;
 	bool verify;
 
 public:
-	SSocket(const char *service, secure::context_t context);
-	SSocket(TCPServer *server, secure::context_t context, size_t size = 536);
-	~SSocket();
+	SSLBuffer(secure::client_t context);
+	SSLBuffer(const TCPServer *server, secure::server_t context, size_t size = 536);
+	~SSLBuffer();
 
 	/**
 	 * Connect a ssl client session to a specific host uri.  If the socket
 	 * was already connected, it is automatically closed first.
-	 * @param host and optional :port we are connecting to.
+	 * @param host we are connecting to.
+	 * @param service to connect to.
 	 * @param size of buffer and tcp fragments.
 	 */
-	void open(const char *host, size_t size = 536);
-
-	void open(TCPServer *server, size_t size = 536);
+	void open(const char *host, const char *service, size_t size = 536);
 
 	void close(void);
 
-	bool flush(void);
-
 	void release(void);
 
 	size_t _push(const char *address, size_t size);
 
 	size_t _pull(char *address, size_t size);
 
-	bool pending(void);
+	bool _flush(void);
 
-	bool issecure(void)
+	bool _pending(void);
+
+	inline bool is_secure(void)
 		{return bio != NULL;};
 };
 
@@ -546,7 +548,7 @@ public:
 /**
  * Convenience type for secure socket.
  */
-typedef	SSocket	ssl_t;
+typedef	SSLBuffer ssl_t;
 
 /**
  * Convenience type for generic digests.
@@ -568,6 +570,52 @@ inline void zerofill(void *addr, size_t size)
 	::memset(addr, 0, size);
 }
 
+#if defined(OLD_STDCPP) || defined(NEW_STDCPP)
+
+/**
+ * Secure socket using std::iostream.  This class is similar to SSLBuffer
+ * but uses the libstdc++ library to stream i/o.  Being based on tcpstream,
+ * it also inherits the character protocol.  Like SSLBuffer, if no context
+ * is given or the handshake fails, then the stream defaults to insecure TCP 
+ * connection behavior.
+ * @author David Sugar <dyfet@gnutelephony.org>
+ */
+class __EXPORT sstream : public tcpstream
+{
+protected:
+	secure::session_t ssl;
+	secure::bufio_t bio;
+	bool server;
+	bool verify;
+
+public:
+	sstream(secure::client_t context);
+	sstream(const TCPServer *server, secure::server_t context, size_t size = 536);
+	~sstream();
+
+	void open(const char *host, const char *service, size_t size = 536);
+
+	void close(void);
+
+	int sync();
+
+	void release(void);
+
+	ssize_t _write(const char *address, size_t size);
+
+	ssize_t _read(char *address, size_t size);
+
+	bool _wait(void);
+
+	inline void flush(void)
+		{sync();}
+
+	inline bool is_secure(void)
+		{return bio != NULL;}
+};
+
+#endif
+
 END_NAMESPACE
 
 #endif
