@@ -128,7 +128,7 @@ public:
 	/**
 	 * Pipe I/O mode.
 	 */
-	typedef	enum {RD = IOBuffer::BUF_RD, WR = IOBuffer::BUF_WR, RDWR = IOBuffer::BUF_RDWR} pmode_t;
+	typedef	enum {RD = BufferProtocol::BUF_RD, WR = BufferProtocol::BUF_WR, RDWR = BufferProtocol::BUF_RDWR} pmode_t;
 
 	/**
 	 * A class to control a process that is piped.  This holds the active
@@ -203,11 +203,16 @@ public:
 	 * class to manage stdio sessions generically in the child process.
 	 * @author David Sugar <dyfet@gnutelephony.org>
 	 */
-	class __EXPORT iobuf : public IOBuffer, protected pipeio
+	class __EXPORT iobuf : public BufferProtocol, private pipeio
 	{
 	protected:
 		friend class shell;
 
+		int ioerror;
+
+		virtual int _err(void) const;
+		virtual void _clear(void);
+
 		virtual size_t _push(const char *address, size_t size);
 		virtual size_t _pull(char *address, size_t size);
 
