@@ -34,6 +34,10 @@
 #include <ucommon/linked.h>
 #endif
 
+#ifndef _UCOMMON_STRING_H_
+#include <ucommon/string.h>
+#endif
+
 extern "C" {
 	struct addrinfo;
 }
@@ -292,11 +296,14 @@ public:
  * addressing, and additional addressing domains (such as Unix domain sockets).
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-class __EXPORT Socket 
+class __EXPORT Socket
 {
 protected:
 	socket_t so;
+	int ioerr;
+	timeout_t iowait;
 
+public:
 	/**
 	 * Get an address list directly.  This is used internally by some derived
 	 * socket types when generic address lists would be invalid.
@@ -314,7 +321,6 @@ protected:
 	 */
 	static void release(struct addrinfo *list);
 
-public:
 	/**
 	 * External definition of fd_set type.  This is used to generate
 	 * select calls which may be wrapped through an internal function
@@ -339,37 +345,47 @@ public:
 
 	public:
 		/**
-		 * Construct a socket address.
+		 * Construct a socket address.  This is used to get an address to
+		 * bind a socket interface to.  The address can be specified as the
+		 * ip address of the interface or as a "hostname".  If a hostname
+		 * is used, then family should be specified for clarity.
+		 * @param family of socket address.  Needed when host names are used.
 		 * @param address or hostname.
-		 * @param family of socket address.  Needed when hostnames are used.
 		 * @param type of socket (stream, dgram, etc).
 		 * @param protocol number of socket.
 		 */
 		address(int family, const char *address, int type = SOCK_STREAM, int protocol = 0);
 
 		/**
-		 * Construct a socket address for an existing socket.
-		 * @param socket to use for family of socket address.
+		 * Construct a socket address for an existing socket.  This can be the
+		 * name of a host or to perform a lookup in a domain for a service.
+		 * Family can be used to restrict the set of results returned, however
+		 * since things like connecto() already filter by family and create
+		 * will use family from the addrinfo, in most cases AF_UNSPEC can be 
+		 * used.  This may be depreciated in favor of the constructor that
+		 * matches a set() method.
+		 * @param family of hosts to filter by or AF_UNSPEC.
 		 * @param hostname or ip address.  The socket family is used for hostnames.
 		 * @param service port or name we are referencing or NULL.
 		 */
-		address(Socket& socket, const char *hostname, const char *service = NULL);
+		address(int family, const char *hostname, const char *service = NULL);
 
 		/**
-		 * Construct a socket address for a socket descriptor.
-		 * @param socket descriptor to use for family.
-		 * @param hostname or address to use.
-		 * @param service port or name we are referencing or NULL.
+		 * Construct a socket address list for a service.
+		 * @param host address for service.
+		 * @param service name or port number.
+		 * @param type of service, stream, dgram, etc.
 		 */
-		address(socket_t socket, const char *hostname, const char *service = NULL);
+		address(const char *host, const char *service, int type = SOCK_STREAM);
 
 		/**
-		 * Construct a socket address from host and service.
+		 * Construct a socket address from host and service.  This is primarily
+		 * used to construct a list of potential service connections by pure 
+		 * port numbers or for host lookup only.
 		 * @param hostname or address to use.
 		 * @param service port or 0.
-		 * @param family of socket address.
 		 */
-		address(const char *hostname, unsigned service = 0, int family = DEFAULT_FAMILY);
+		address(const char *hostname, unsigned service = 0);
 
 		/**
 		 * Construct an empty address.
@@ -401,7 +417,7 @@ public:
 		struct sockaddr *get(int family) const;
 
 		/**
-		 * Get the family of a socket address by first entry.
+		 * Get the family of the first member in a list of services.
 		 * @return family of first socket address or 0 if none.
 		 */
 		int getfamily(void) const;
@@ -463,24 +479,22 @@ public:
 		 * Set the host addresses to form a new list.
 		 * @param hostname or address to resolve.
 		 * @param service name or port number, or NULL if not used.
-		 * @param family of hostname.
-		 * @param type of socket.
+		 * @param type of socket (stream or dgram) to filter list by.
 		 */
-		void set(const char *hostname, const char *service = NULL, int family = 0, int type = SOCK_STREAM);
+		void set(const char *hostname, const char *service = NULL, int type = SOCK_STREAM);
 
 		/**
 		 * Append additional host addresses to our list.
 		 * @param hostname or address to resolve.
 		 * @param service name or port number, or NULL if not used.
-		 * @param family of hostname.
-		 * @param type of socket.
+		 * @param type of socket (stream or dgram).
 		 */
-		void add(const char *hostname, const char *service = NULL, int family = 0, int type = SOCK_STREAM);
+		void add(const char *hostname, const char *service = NULL, int type = SOCK_STREAM);
 
 		/**
 		 * Set an entry for host binding.
-		 * @param address or hostname.
 		 * @param family of socket address.  Needed when hostnames are used.
+		 * @param address or hostname.	
 		 * @param type of socket (stream, dgram, etc).
 		 * @param protocol number of socket.
 		 */
@@ -492,6 +506,20 @@ public:
 		 */
 		void add(sockaddr *address);
 
+        /**
+         * Insert unique members from another socket address list to ours.
+         * @param address list to insert into list.
+         * @return count of addresses added.
+         */
+        unsigned insert(struct addrinfo *address);
+
+        /**
+         * Remove members from another socket address list from ours.
+         * @param address list to remove from list.
+         * @return count of addresses removed.
+         */
+        unsigned remove(struct addrinfo *address);
+
 		/**
 		 * Remove an individual socket address from our address list.
 		 * @param address to remove.
@@ -507,22 +535,6 @@ public:
 		 */
 		bool insert(struct sockaddr *address);
 
-		/**
-		 * Insert unique members from another socket address list to ours.
-		 * @param address list to insert into list.
-		 * @param family to scan for or 0 for all.
-		 * @return count of addresses added.
-		 */
-		unsigned insert(struct addrinfo *address, int family = 0);
-
-		/**
-		 * Remove members from another socket address list from ours.
-		 * @param address list to remove from list.
-		 * @param family to scan for or 0 for all.
-		 * @return count of addresses removed.
-		 */
-		unsigned remove(struct addrinfo *address, int family = 0);
-
 		/**
 		 * Copy an existing addrinfo into our object.  This is also used
 		 * to support the copy constructor.
@@ -540,9 +552,8 @@ public:
 		 * Set a socket address from host and service.
 		 * @param hostname or address to use.
 		 * @param service port or 0.
-		 * @param family of socket address.
 		 */
-		void set(const char *hostname, unsigned service = 0, int family = DEFAULT_FAMILY);
+		void set(const char *hostname, unsigned service = 0);
 
 		/**
 		 * Duplicate a socket address.
@@ -603,15 +614,15 @@ public:
 	Socket(int family, int type, int protocol = 0);
 
 	/**
-	 * Create a bound socket.
-	 * @param address to bind or "*" for all
+	 * Create a bound socket.  If one wishes to listen for connections on
+	 * a protocol, then ListenSocket should be used instead.
+	 * @param address to bind or "*" for all.
 	 * @param port number of service to bind.
 	 * @param family to bind as.
 	 * @param type of socket to bind (stream, udp, etc).
 	 * @param protocol of socket to bind.
-	 * @param backlog for listening socket.
 	 */
-	Socket(const char *address, const char *port, int family = AF_UNSPEC, int type = 0, int protocol = 0, int backlog = 0);
+	Socket(const char *address, const char *port, int family = AF_UNSPEC, int type = 0, int protocol = 0);
 
 	/**
 	 * Shutdown, close, and destroy socket.
@@ -634,6 +645,12 @@ public:
 	 */
 	void release(void);
 
+	/**
+	 * Get error code.
+	 */
+	inline int err(void) const
+		{return ioerr;}
+
 	/**
 	 * See the number of bytes in the receive queue.
 	 * @param value to test for.
@@ -843,35 +860,38 @@ public:
 	 * For TCP (and DCCP) sockets, the entire list may be tried.  For UDP, 
 	 * connect is only a state and the first valid entry in the list is used.
 	 * @param list of addresses to connect to.
-	 * @return 0 on success, -1 on error.
+	 * @return 0 on success or error.
 	 */
-	inline int connectto(struct addrinfo *list)
-		{return connectto(so, list);};
+	int connectto(struct addrinfo *list);
 	
 	/**
 	 * Disconnect a connected socket.  Depending on the implementation, this
 	 * might be done by connecting to AF_UNSPEC, connecting to a 0 address,
 	 * or connecting to self.
-	 * @return 0 on success, -1 on error.
+	 * @return 0 on success or error.
 	 */
-	inline int disconnect(void)
-		{return disconnect(so);};
+	int disconnect(void);
 
 	/**
 	 * Join socket to multicast group.
 	 * @param list of groups to join.
 	 * @return 0 on success, -1 on error.
 	 */
-	inline int join(struct addrinfo *list)
-		{return join(so, list);};
+	int join(struct addrinfo *list);
 
 	/**
 	 * Drop socket from multicast group.
 	 * @param list of groups to drop.
 	 * @return 0 on success, -1 on error.
 	 */
-	inline int drop(struct addrinfo *list)
-		{return drop(so, list);};
+	int drop(struct addrinfo *list);
+
+	/**
+	 * Socket i/o timer setting.
+	 * @param timeout to wait, inf for blocking, 0 pure non-blocking.
+	 * @return 0 on success or error code.
+	 */
+	int wait(timeout_t timeout = Timer::inf);
 
 	/**
 	 * Peek at data waiting in the socket receive buffer.
@@ -881,26 +901,6 @@ public:
 	 */
 	size_t peek(void *data, size_t number) const;
 
-	/**
-	 * Read data from the socket receive buffer.  This is a virtual so that
-	 * the ssl layer can override the core get method.
-	 * @param data pointer to save data in.
-	 * @param number of bytes to read.
-	 * @param address of peer data was received from.
-	 * @return number of bytes actually read, 0 if none, -1 if error.
-	 */
-	virtual ssize_t get(void *data, size_t number, struct sockaddr_storage *address = NULL);
-
-	/**
-	 * Write data to the socket send buffer.  This is a virtual so that the ssl
-	 * layer can override the core put method.
-	 * @param data pointer to write data from.
-	 * @param number of bytes to write.
-	 * @param address of peer to send data to if not connected.
-	 * @return number of bytes actually sent, 0 if none, -1 if error.
-	 */
-	virtual ssize_t put(const void *data, size_t number, struct sockaddr *address = NULL);
-
 	/**
 	 * Read data from the socket receive buffer.  This will be used in abi 4.
 	 * @param data pointer to save data in.
@@ -908,8 +908,7 @@ public:
 	 * @param address of peer data was received from.
 	 * @return number of bytes actually read, 0 if none, -1 if error.
 	 */
-	inline ssize_t readfrom(void *data, size_t number, struct sockaddr_storage *address = NULL)
-		{return get(data, number, address);};
+	size_t readfrom(void *data, size_t number, struct sockaddr_storage *address = NULL);
 
 	/**
 	 * Write data to the socket send buffer.  This will be used in abi 4.
@@ -918,20 +917,41 @@ public:
 	 * @param address of peer to send data to if not connected.
 	 * @return number of bytes actually sent, 0 if none, -1 if error.
 	 */
-	inline ssize_t writeto(const void *data, size_t number, struct sockaddr *address = NULL)
-		{return put(data, number, address);};
+	size_t writeto(const void *data, size_t number, struct sockaddr *address = NULL);
 
 	/**
 	 * Read a newline of text data from the socket and save in NULL terminated
 	 * string.  This uses an optimized I/O method that takes advantage of
-	 * socket peeking.  As such, it has to be rewritten to be used in a ssl
-	 * layer socket.
+	 * socket peeking.  This presumes a connected socket on a streamble
+	 * protocol.  Because the trailing newline is dropped, the return size
+	 * may be greater than the string length.  If there was no data read
+	 * because of eof of data, an error has occured, or timeout without
+	 * input, then 0 will be returned.
 	 * @param data to save input line.
 	 * @param size of input line buffer.
-	 * @param timeout to wait for a complete input line.
-	 * @return number of bytes read, 0 if none, -1 if error.
+	 * @return number of bytes read, 0 if none, err() has error.
 	 */
-	virtual ssize_t gets(char *data, size_t size, timeout_t timeout = Timer::inf);
+	size_t readline(char *data, size_t size);
+
+	/**
+	 * Print formatted string to socket.
+	 * @param format string.
+	 * @return number of bytes sent.
+	 */
+	size_t printf(const char *format, ...) __PRINTF(2,3);
+
+	/**
+	 * Read a string of input from the socket and strip trailing newline.  
+	 * This uses an optimized I/O method that takes advantage of
+	 * socket peeking.  This presumes a connected socket on a streamble
+	 * protocol.  Because the trailing newline is dropped, the return size
+	 * may be greater than the string length.  If there was no data read
+	 * because of eof of data, an error has occured, or timeout without
+	 * input, then 0 will be returned.
+	 * @param buffer to save input line.
+	 * @return number of bytes read, 0 if none, err() has error.
+	 */
+	size_t readline(string& buffer);
 
 	/**
 	 * Read a newline of text data from the socket and save in NULL terminated
@@ -954,22 +974,14 @@ public:
 	 */
 	static ssize_t printf(socket_t socket, const char *format, ...) __PRINTF(2,3);
 
-	/**
-	 * Write a null terminated string to the socket.
-	 * @param string to write.
-	 * @return number of bytes sent, 0 if none, -1 if error.
-	 */
-	ssize_t puts(const char *string);
-
 	/**
 	 * Write a null terminated string to the socket.  This exists because
 	 * we messed up consistency with the original puts() method.  In the
 	 * future there will be a single puts() that has a NULL default.
 	 * @param string to write.
-	 * @param address to write to.
 	 * @return number of bytes sent, 0 if none, -1 if error.
 	 */
-	ssize_t puts(const char *string, struct sockaddr *address);
+	size_t writes(const char *string);
 
 	/**
 	 * Test if socket is valid.
@@ -1286,10 +1298,9 @@ public:
 	 * @param family to select or AF_UNSPEC
 	 * @param type of socket to create.
 	 * @param protocol of socket to create.
-	 * @param backlog for listener.
 	 * @return socket descriptor created or INVALID_SOCKET.
 	 */
-	static socket_t create(const char *iface, const char *service, int family = AF_UNSPEC, int type = 0, int protocol = 0, int backlog = 0);
+	static socket_t create(const char *iface, const char *service, int family = AF_UNSPEC, int type = 0, int protocol = 0);
 
 	/**
 	 * Create a connected socket for a service.
@@ -1312,8 +1323,8 @@ public:
 	 * @param address to send to or NULL if connected.
 	 * @return number of bytes send, -1 if error.
 	 */
-	inline static ssize_t writeto(Socket& socket, const char *buffer, size_t size, struct sockaddr *address)
-		{return socket.put(buffer, size, address);};
+	inline static size_t writeto(Socket& socket, const char *buffer, size_t size, struct sockaddr *address)
+		{return socket.writeto(buffer, size, address);};
 	
 	/**
 	 * receive from socket object.
@@ -1323,8 +1334,8 @@ public:
 	 * @param address receiving from or NULL if connected.
 	 * @return number of bytes received, -1 if error.
 	 */
-	inline static ssize_t readfrom(Socket& socket, char *buffer, size_t size, struct sockaddr_storage *address)
-		{return socket.get(buffer, size, address);};
+	inline static size_t readfrom(Socket& socket, char *buffer, size_t size, struct sockaddr_storage *address)
+		{return socket.readfrom(buffer, size, address);};
 
 	/**
 	 * Connect a socket.
@@ -1534,27 +1545,6 @@ public:
 	 */
 	static void v4mapping(bool enable);
 
-	/**
-	 * Convert socket into FILE handle for reading or writing.
-	 * @param socket descriptor to convert.
-	 * @param mode of access, true for write mode.
-	 * @return file handle to use.
-	 */
-	static FILE *open(socket_t socket, bool mode);
-
-	/**
-	 * Convert socket into FILE handle for reading and writing.
-	 * @param socket descriptor to convert.
-	 * @return file handle to use.
-	 */
-	static FILE *open(socket_t socket);
-
-	/**
-	 * Cleanly close a connected socket descriptor mapped to a file handle.
-	 * @param file handle to close.
-	 */
-	static void close(FILE *file);
-
 	/**
 	 * Return error code of last socket operation,
 	 * @return errno style error code.
@@ -1673,16 +1663,30 @@ public:
 	 * @param address to bind on or "*" for all.
 	 * @param service port to bind listener.
 	 * @param backlog size for buffering pending connections.
+	 * @param family of socket.
+	 * @param type of socket.
 	 * @param protocol for socket if not TCPIP.
 	 */
-	ListenSocket(const char *address, const char *service, unsigned backlog = 5, int protocol = 0);
+	ListenSocket(const char *address, const char *service, unsigned backlog = 5, int family = AF_UNSPEC, int type = 0, int protocol = 0);
+
+	/**
+	 * Create a listen socket directly.
+	 * @param address to bind on or "*" for all.
+	 * @param service port to bind listener.
+	 * @param backlog size for buffering pending connections.
+	 * @param family of socket.
+	 * @param type of socket.
+	 * @param protocol for socket if not TCPIP.
+	 * @return bound and listened to socket.
+	 */
+	static socket_t create(const char *address, const char *service, unsigned backlog = 5, int family = AF_UNSPEC, int type = 0, int protocol = 0);
 
 	/**
 	 * Accept a socket connection.
 	 * @param address to save peer connecting.  
 	 * @return socket descriptor of connected socket.
 	 */
-	socket_t accept(struct sockaddr_storage *address = NULL);
+	socket_t accept(struct sockaddr_storage *address = NULL) const;
 
 	/**
 	 * Wait for a pending connection.
@@ -1715,6 +1719,24 @@ public:
 
 };
 
+/**
+ * A generic tcp server class.  This saves the service id tag so that it
+ * can be propagated.
+ * @author David Sugar <dyfet@gnutelephony.org>
+ */
+class __EXPORT TCPServer : public ListenSocket
+{
+public:
+	/**
+	 * Create and bind a tcp server.  This mostly is used to preserve the
+	 * service tag for TCP Socket when derived from a server instance.
+	 * @param service tag to use.
+	 * @param address of interface to bind or "*" for all.
+	 * @param backlog size for pending connections.
+	 */
+	TCPServer(const char *address, const char *service, unsigned backlog = 5);
+};
+
 /**
  * A convenience class for socket.
  */
@@ -1767,6 +1789,10 @@ inline bool eq(struct sockaddr_storage *s1, struct sockaddr_storage *s2)
 inline bool ieq(struct sockaddr *s1, struct sockaddr *s2)
 	{return Socket::equalhost(s1, s2);}
 
+String str(Socket& so, strsize_t size);
+
+typedef	TCPServer	tcpserv_t;
+
 END_NAMESPACE
 
 #endif
