@@ -35,16 +35,16 @@
 #ifndef	_UCOMMON_STRING_H_
 #define	_UCOMMON_STRING_H_
 
-#ifndef	_UCOMMON_MEMORY_H_
-#include <ucommon/memory.h>
+#ifndef	_UCOMMON_CONFIG_H_
+#include <ucommon/platform.h>
 #endif
 
-#ifndef	_UCOMMON_TIMERS_H_
-#include <ucommon/timers.h>
+#ifndef _UCOMMON_PROTOCOLS_H_
+#include <ucommon/protocols.h>
 #endif
 
-#ifndef	_UCOMMON_SOCKET_H_
-#include <ucommon/socket.h>
+#ifndef _UCOMMON_OBJECT_H_
+#include <ucommon/object.h>
 #endif
 
 #include <stdio.h>
@@ -62,8 +62,6 @@ NAMESPACE_UCOMMON
  */
 typedef	unsigned short strsize_t;
 
-class StringFormat;
-
 /**
  * A copy-on-write string class that operates by reference count.  This string
  * class anchors a counted object that is managed as a copy-on-write
@@ -77,8 +75,6 @@ class StringFormat;
 class __EXPORT string : public Object
 {
 protected:
-	friend class StringFormat;
-
 	/**
 	 * This is an internal class which contains the actual string data
 	 * along with some control fields.  The string can be either NULL
@@ -134,12 +130,6 @@ public:
 		 */
 		void set(strsize_t offset, const char *text, strsize_t size);
 
-		/**
-		 * Set our string from a string formatting object.
-		 * @param format object to set from.
-		 */
-		void set(const StringFormat& format);
-
 		/**
 		 * Set our string from null terminated text up to our allocated size.
 		 * @param text to set from.
@@ -152,12 +142,6 @@ public:
 		 */
 		void add(const char *text);
 
-		/**
-		 * Append text to string buffer from formatting object.
-		 * @param format object to add from.
-		 */
-		void add(const StringFormat &format);
-
 		/**
 		 * Append a single character to our string buffer.
 		 * @param character to append.
@@ -256,12 +240,6 @@ public:
 	 */
 	string();
 
-	/**
-	 * Create a string from a formatting object.
-	 * @param format object to use in creating string.
-	 */
-	string(const StringFormat& format);
-
 	/**
 	 * Create a string from a long integer.
 	 * @param value to convert to string.
@@ -395,12 +373,6 @@ public:
 	 */
 	void set(const char *text);
 
-	/**
-	 * Set string object to text of a formatted object.
-	 * @param format object to use.
-	 */
-	void set(const StringFormat& format);
-
 	/**
 	 * Set a portion of the string object at a specified offset to a text
 	 * string.
@@ -428,13 +400,6 @@ public:
 	 */
 	void rset(const char *text, char overflow, strsize_t offset, strsize_t size = 0);
 
-	/**
-	 * Append formatted object to our string buffer.
-	 * @param format object to append.
-	 */
-	void add(const StringFormat& format);
-
-
 	/**
 	 * Append null terminated text to our string buffer.
 	 * @param text to append.
@@ -715,13 +680,6 @@ public:
 	 */
 	string& operator^=(const string& object);
 
-	/**
-	 * Create new cow instance and assign value from formatted string object.
-	 * @param format object to assign from.
-	 * @return our object for expression use.
-	 */
-	string& operator^=(const StringFormat& format);
-
 	/**
 	 * Create new cow instance and assign value from null terminated text.
 	 * @param text to assign from.
@@ -736,13 +694,6 @@ public:
 	 */
 	string& operator+(const char *text);
 
-	/**
-	 * Concatenate string formatted object to our object.  This creates a new
-	 * copy-on-write instance to hold the concatenated string.
-	 * @param format object to concatenate.
-	 */
-	string& operator+(const StringFormat& format);
-
 	/**
 	 * Concatenate null terminated text to our object.  This directly
 	 * appends the text to the string buffer and does not resize the
@@ -751,14 +702,6 @@ public:
 	 */
 	string& operator&(const char *text);
 
-	/**
-	 * Concatenate formatted string object to our object.  This directly
-	 * appends the text to the string buffer and does not resize the
-	 * object if the existing cstring allocation space is fully used.
-	 * @param format object to concatenate.
-	 */
-	string& operator&(const StringFormat& format);
-
 	/**
 	 * Assign our string with the cstring of another object.  If we had
 	 * an active string reference, it is released.  The object now has
@@ -767,12 +710,6 @@ public:
 	 */
 	string& operator=(const string& object);
 
-	/**
-	 * Assign our string from a string formatting object.
-	 * @param format object to assign from.
-	 */
-	string& operator=(const StringFormat& format);
-
 	/**
 	 * Assign text to our existing buffer.  This performs a set method.
 	 * @param text to assign from.
@@ -902,80 +839,6 @@ public:
 	 */
 	static strsize_t printf(string& object, const char *format, ...) __PRINTF(2, 3);
 
-	/**
-	 * Read arbitrary binary data from socket into a string object.  The 
-	 * total number of bytes that may be read is based on the allocated
-	 * size of the object.
-	 * @param socket to read from.
-	 * @param object to save read data.
-	 * @return number of bytes read.
-	 */
-	static int read(Socket& socket, string& object);
-	
-	/**
-	 * Write the string object to a socket.
-	 * @param socket to write to.
-	 * @param object to get data from.
-	 * @return number of bytes written.
-	 */
-	static int write(Socket& socket, string& object);
-
-	/**
-	 * Read arbitrary binary data from a file into a string object.  The 
-	 * total number of bytes that may be read is based on the allocated
-	 * size of the object.
-	 * @param file to read from.
-	 * @param object to save read data.
-	 * @return number of bytes read.
-	 */
-	static int read(FILE *file, string& object);
-
-	/**
-	 * Write the string object to a file.
-	 * @param file to write to.
-	 * @param object to get data from.
-	 * @return number of bytes written.
-	 */
-	static int write(FILE *file, string& object); 
-
-	/**
-	 * Read a line of text input from a socket into the object.  The 
-	 * maximum number of bytes that may be read is based on the currently
-	 * allocated size of the object.
-	 * @param socket to read from.
-	 * @param object to save read data.
-	 * @return false if end of file.
-	 */
-	static bool getline(Socket& socket, string& object);
-
-	/**
-	 * Write string as a line of text data to a socket.  A newline will be 
-	 * appended to the end.
-	 * @param socket to print to.
-	 * @param object to get text line from.
-	 * @return true if successful.
-	 */
-	static bool putline(Socket& socket, string& object);
-
-	/**
-	 * Read a line of text input from a file into the object.  The 
-	 * maximum number of bytes that may be read is based on the currently
-	 * allocated size of the object.
-	 * @param file to read from.
-	 * @param object to save read data.
-	 * @return false if end of file.
-	 */
-	static bool getline(FILE *file, string& object);
-
-	/**
-	 * Write string as a line of text data to a file.  A newline will be 
-	 * appended to the end.
-	 * @param file to print to.
-	 * @param object to get text line to put into file.
-	 * @return true if successful.
-	 */
-	static bool putline(FILE *file, string& object);
-
 	/**
 	 * Swap the cstring references between two strings.
 	 * @param object1 to swap.
@@ -1556,36 +1419,7 @@ public:
 	 * @param size of string to allocate.  Automatically adds control size.
 	 * @param fill character for fixed field strings.
 	 */
-	static memstring *create(mempager *pager, strsize_t size, char fill = 0);
-};
-
-/**
- * A string conversion class for use as a base class in objects which can
- * transform themselves into string representations.  This can be used
- * together with the string class for automatic conversions.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-class __EXPORT StringFormat
-{
-protected:
-	friend class string;
-	friend class string::cstring;
-
-	virtual ~StringFormat();
-
-	/**
-	 * Convert derived object into a string buffer.  This may be a public
-	 * method in a derived class.
-	 * @param buffer to save representation in.
-	 * @param size of buffer to use.
-	 */
-	virtual void put(char *buffer, size_t size) const = 0;
-
-	/**
-	 * Get maximum space that might be needed for string representation.
-	 * @return space needed.
-	 */
-	virtual strsize_t getStringSize(void) const = 0;
+	static memstring *create(MemoryProtocol *pager, strsize_t size, char fill = 0);
 };
 
 /**
@@ -1683,6 +1517,17 @@ public:
 		{return S;};
 };
 
+/**
+ * A convenience type for string.
+ */
+typedef	string string_t;
+
+/**
+ * A convenience type when mixing std::string in old compilers that are bad
+ * with namespaces...
+ */
+typedef string String;
+
 /**
  * A string class that has a predefined string buffer.  The string class
  * and buffer are allocated together as one object.  This allows one to use
@@ -1749,17 +1594,6 @@ extern "C" inline int strnicmp(const char *string1, const char *string2, size_t
 
 #endif
 
-/**
- * A convenience type for string.
- */
-typedef	string string_t;
-
-/**
- * A convenience type when mixing std::string in old compilers that are bad
- * with namespaces...
- */
-typedef string String;
-
 /**
  * Compare two null terminated strings if equal.
  * @param s1 string to compare.
@@ -1840,6 +1674,8 @@ inline String str(unsigned long value)
 inline String str(double value)
 	{String temp(40, "%f", value); return temp;} 
 
+String str(CharacterProtocol& cp, strsize_t size);
+
 END_NAMESPACE
 
 #endif
