@@ -73,15 +73,12 @@ protected:
  * returned for reuse.  
   * @author David Sugar <dyfet@gnutelephony.org>
  */
-class __EXPORT PagerReuse : protected ReusableAllocator
+class __EXPORT PagerReuse : protected MemoryRedirect, protected ReusableAllocator
 {
 private:
-	mempager *pager;
 	unsigned limit, count;
 	size_t osize;
 
-	ReusableObject *alloc(void);
-
 protected:
 	PagerReuse(mempager *pager, size_t objsize, unsigned count);
 	~PagerReuse();
@@ -447,6 +444,9 @@ public:
 	inline T *get(int index)
 		{return static_cast<T *>(Vector::get(index));};
 
+	inline T& operator[](int index)
+		{return static_cast<T *>(Vector::get(index));};
+
 	/**
 	 * Retrieve a typed member of the vector directly.
 	 * @param position to retrieve object from.
@@ -702,6 +702,9 @@ public:
 	inline T *get(int index)
 		{return static_cast<T *>(Vector::get(index));};
 
+	inline T& operator[](int index)
+		{return static_cast<T*>(Vector::get(index));};
+
 	/**
 	 * Retrieve a typed member of the fixed vector directly.
 	 * @param position to retrieve object from.
